@@ -16,6 +16,7 @@ from photutils.background.core import (BiweightLocationBackground,
                                        ModeEstimatorBackground,
                                        SExtractorBackground, StdBackgroundRMS)
 from photutils.datasets import make_noise_image
+from photutils.utils._stats import nanmean
 
 BKG = 0.0
 STD = 0.5
@@ -80,10 +81,7 @@ def test_background_axis(bkg_class):
     bkg = bkg_class(sigma_clip=SIGMA_CLIP)
 
     bkg_arr = bkg.calc_background(DATA, axis=0)
-    bkgi = []
-    for i in range(100):
-        bkgi.append(bkg.calc_background(DATA[:, i]))
-    bkgi = np.array(bkgi)
+    bkgi = np.array([bkg.calc_background(DATA[:, i]) for i in range(100)])
     assert_allclose(bkg_arr, bkgi)
 
     bkg_arr = bkg.calc_background(DATA, axis=1)
@@ -107,6 +105,62 @@ def test_sourceextrator_background_skew():
     assert_allclose(bkg.calc_background(data), np.median(data))
 
 
+@pytest.mark.parametrize('bkg_class', BKG_CLASS)
+def test_background_ndim(bkg_class):
+    data1 = np.ones((1, 100, 100))
+    data2 = np.ones((1, 100 * 100))
+    data3 = np.ones((1, 1, 100 * 100))
+    data4 = np.ones((1, 1, 1, 100 * 100))
+
+    bkg = bkg_class(sigma_clip=None)
+    val = bkg(data1, axis=None)
+    assert np.ndim(val) == 0
+    val = bkg(data1, axis=(1, 2))
+    assert val.shape == (1,)
+    val = bkg(data1, axis=-1)
+    assert val.shape == (1, 100)
+    val = bkg(data2, axis=-1)
+    assert val.shape == (1,)
+    val = bkg(data3, axis=-1)
+    assert val.shape == (1, 1)
+    val = bkg(data4, axis=-1)
+    assert val.shape == (1, 1, 1)
+    val = bkg(data4, axis=(2, 3))
+    assert val.shape == (1, 1)
+    val = bkg(data4, axis=(1, 2, 3))
+    assert val.shape == (1,)
+    val = bkg(data4, axis=(0, 1, 2))
+    assert val.shape == (10000,)
+
+
+@pytest.mark.parametrize('bkgrms_class', RMS_CLASS)
+def test_background_rms_ndim(bkgrms_class):
+    data1 = np.ones((1, 100, 100))
+    data2 = np.ones((1, 100 * 100))
+    data3 = np.ones((1, 1, 100 * 100))
+    data4 = np.ones((1, 1, 1, 100 * 100))
+
+    bkgrms = bkgrms_class(sigma_clip=None)
+    val = bkgrms(data1, axis=None)
+    assert np.ndim(val) == 0
+    val = bkgrms(data1, axis=(1, 2))
+    assert val.shape == (1,)
+    val = bkgrms(data1, axis=-1)
+    assert val.shape == (1, 100)
+    val = bkgrms(data2, axis=-1)
+    assert val.shape == (1,)
+    val = bkgrms(data3, axis=-1)
+    assert val.shape == (1, 1)
+    val = bkgrms(data4, axis=-1)
+    assert val.shape == (1, 1, 1)
+    val = bkgrms(data4, axis=(2, 3))
+    assert val.shape == (1, 1)
+    val = bkgrms(data4, axis=(1, 2, 3))
+    assert val.shape == (1,)
+    val = bkgrms(data4, axis=(0, 1, 2))
+    assert val.shape == (10000,)
+
+
 @pytest.mark.parametrize('rms_class', RMS_CLASS)
 def test_background_rms(rms_class):
     bkgrms = rms_class(sigma_clip=SIGMA_CLIP)
@@ -119,10 +173,8 @@ def test_background_rms_axis(rms_class):
     bkgrms = rms_class(sigma_clip=SIGMA_CLIP)
 
     rms_arr = bkgrms.calc_background_rms(DATA, axis=0)
-    rmsi = []
-    for i in range(100):
-        rmsi.append(bkgrms.calc_background_rms(DATA[:, i]))
-    rmsi = np.array(rmsi)
+    rmsi = np.array([bkgrms.calc_background_rms(DATA[:, i])
+                     for i in range(100)])
     assert_allclose(rms_arr, rmsi)
 
     rms_arr = bkgrms.calc_background_rms(DATA, axis=1)
@@ -166,7 +218,7 @@ def test_background_masked(bkg_class):
     # test masked array with masked=False with axis
     bkgval2 = bkg.calc_background(data, masked=False, axis=1)
     assert not np.ma.isMaskedArray(bkgval2)
-    assert_allclose(np.nanmean(bkgval2), BKG, atol=0.01)
+    assert_allclose(nanmean(bkgval2), BKG, atol=0.01)
 
 
 @pytest.mark.parametrize('rms_class', RMS_CLASS)
@@ -186,7 +238,7 @@ def test_background_rms_masked(rms_class):
     # test masked array with masked=False with axis
     rms3 = bkgrms.calc_background_rms(data, masked=False, axis=1)
     assert not np.ma.isMaskedArray(rms3)
-    assert_allclose(np.nanmean(rms3), STD, atol=0.01)
+    assert_allclose(nanmean(rms3), STD, atol=0.01)
 
 
 @pytest.mark.parametrize('bkg_class', BKG_CLASS)
@@ -215,13 +267,15 @@ def test_background_rms_units(rms_class):
 
 @pytest.mark.parametrize('bkg_class', BKG_CLASS)
 def test_background_invalid_sigmaclip(bkg_class):
-    with pytest.raises(TypeError):
+    match = 'sigma_clip must be an astropy SigmaClip instance or None'
+    with pytest.raises(TypeError, match=match):
         bkg_class(sigma_clip=3)
 
 
 @pytest.mark.parametrize('rms_class', RMS_CLASS)
 def test_background_rms_invalid_sigmaclip(rms_class):
-    with pytest.raises(TypeError):
+    match = 'sigma_clip must be an astropy SigmaClip instance or None'
+    with pytest.raises(TypeError, match=match):
         rms_class(sigma_clip=3)
 
 
